# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: protobuf/server.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x15protobuf/server.proto\x12\x06Server\"\xc2\x04\n\rServerCommand\x12\x0c\n\x04UUID\x18\x01 \x01(\t\x12\x15\n\x0blog_message\x18\x02 \x01(\tH\x00\x12\x18\n\x0e\x63\x61pture_screen\x18\x03 \x01(\x08H\x00\x12\x33\n\x12\x61\x64\x64_custom_control\x18\x04 \x01(\x0b\x32\x15.Server.CustomControlH\x00\x12\'\n\nset_camera\x18\x05 \x01(\x0b\x32\x11.Server.SetCameraH\x00\x12\'\n\nadd_object\x18\x06 \x01(\x0b\x32\x11.Server.AddObjectH\x00\x12;\n\x15set_key_event_handler\x18\x07 \x01(\x0b\x32\x1a.Server.SetKeyEventHandlerH\x00\x12-\n\rremove_object\x18\x08 \x01(\x0b\x32\x14.Server.RemoveObjectH\x00\x12<\n\x15remove_custom_control\x18\t \x01(\x0b\x32\x1b.Server.RemoveCustomControlH\x00\x12\x36\n\x12set_custom_control\x18\n \x01(\x0b\x32\x18.Server.SetCustomControlH\x00\x12\x14\n\nset_enable\x18\x0b \x01(\x08H\x00\x12\x1a\n\x10get_camera_state\x18\x0c \x01(\x08H\x00\x12L\n\x1eset_camera_state_event_handler\x18\r \x01(\x0b\x32\".Server.SetCameraStateEventHandlerH\x00\x42\t\n\x07\x43ommand\"\xbf\x06\n\rCustomControl\x12.\n\x06slider\x18\x01 \x01(\x0b\x32\x1c.Server.CustomControl.SliderH\x00\x12\x32\n\x08\x63heckbox\x18\x02 \x01(\x0b\x32\x1e.Server.CustomControl.CheckBoxH\x00\x12\x30\n\x07textbox\x18\x03 \x01(\x0b\x32\x1d.Server.CustomControl.TextBoxH\x00\x12\x34\n\tselectbox\x18\x04 \x01(\x0b\x32\x1f.Server.CustomControl.SelectBoxH\x00\x12.\n\x06\x62utton\x18\x05 \x01(\x0b\x32\x1c.Server.CustomControl.ButtonH\x00\x12\x39\n\x0c\x63olor_picker\x18\x06 \x01(\x0b\x32!.Server.CustomControl.ColorPickerH\x00\x12.\n\x06\x66older\x18\x07 \x01(\x0b\x32\x1c.Server.CustomControl.FolderH\x00\x1a\x62\n\x06Slider\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0b\n\x03min\x18\x02 \x01(\x02\x12\x0b\n\x03max\x18\x03 \x01(\x02\x12\x0c\n\x04step\x18\x04 \x01(\x02\x12\x12\n\ninit_value\x18\x05 \x01(\x02\x12\x0e\n\x06parent\x18\x06 \x01(\t\x1a<\n\x08\x43heckBox\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x12\n\ninit_value\x18\x02 \x01(\x08\x12\x0e\n\x06parent\x18\x03 \x01(\t\x1a;\n\x07TextBox\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x12\n\ninit_value\x18\x02 \x01(\t\x12\x0e\n\x06parent\x18\x03 \x01(\t\x1aL\n\tSelectBox\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05items\x18\x02 \x03(\t\x12\x12\n\ninit_value\x18\x03 \x01(\t\x12\x0e\n\x06parent\x18\x04 \x01(\t\x1a&\n\x06\x42utton\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0e\n\x06parent\x18\x02 \x01(\t\x1a?\n\x0b\x43olorPicker\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x12\n\ninit_value\x18\x02 \x01(\t\x12\x0e\n\x06parent\x18\x03 \x01(\t\x1a&\n\x06\x46older\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0e\n\x06parent\x18\x02 \x01(\tB\t\n\x07\x43ontrol\"A\n\x13RemoveCustomControl\x12\r\n\x03\x61ll\x18\x01 \x01(\x08H\x00\x12\x11\n\x07\x62y_uuid\x18\x02 \x01(\tH\x00\x42\x08\n\x06Object\"\xef\x02\n\tSetCamera\x12\x19\n\x0fperspective_fov\x18\x01 \x01(\x02H\x00\x12%\n\x1borthographic_frustum_height\x18\x02 \x01(\x02H\x00\x12#\n\x08position\x18\x03 \x01(\x0b\x32\x0f.Server.VecXYZfH\x00\x12!\n\x06target\x18\x04 \x01(\x0b\x32\x0f.Server.VecXYZfH\x00\x12&\n\x04roll\x18\x05 \x01(\x0b\x32\x16.Server.SetCamera.RollH\x00\x12\x13\n\troll_lock\x18\x06 \x01(\x08H\x00\x12,\n\x04mode\x18\x07 \x01(\x0e\x32\x1c.Server.SetCamera.CameraModeH\x00\x1a\x32\n\x04Roll\x12\r\n\x05\x61ngle\x18\x01 \x01(\x02\x12\x1b\n\x02up\x18\x02 \x01(\x0b\x32\x0f.Server.VecXYZf\"/\n\nCameraMode\x12\x0f\n\x0bPERSPECTIVE\x10\x00\x12\x10\n\x0cORTHOGRAPHIC\x10\x01\x42\x08\n\x06\x43\x61mera\"\xfb\x07\n\tAddObject\x12-\n\x08line_set\x18\x01 \x01(\x0b\x32\x19.Server.AddObject.LineSetH\x00\x12\x33\n\x0bpoint_cloud\x18\x02 \x01(\x0b\x32\x1c.Server.AddObject.PointCloudH\x00\x12,\n\x07overlay\x18\x03 \x01(\x0b\x32\x19.Server.AddObject.OverlayH\x00\x12&\n\x04mesh\x18\x04 \x01(\x0b\x32\x16.Server.AddObject.MeshH\x00\x12(\n\x05image\x18\x05 \x01(\x0b\x32\x17.Server.AddObject.ImageH\x00\x1a\x81\x01\n\x07LineSet\x12\x1f\n\x06points\x18\x01 \x03(\x0b\x32\x0f.Server.VecXYZf\x12\x12\n\nfrom_index\x18\x02 \x03(\x05\x12\x10\n\x08to_index\x18\x03 \x03(\x05\x12\x1f\n\x06\x63olors\x18\x04 \x03(\x0b\x32\x0f.Server.VecRGBf\x12\x0e\n\x06widths\x18\x05 \x03(\x02\x1a\x32\n\nPointCloud\x12\x10\n\x08pcd_data\x18\x01 \x01(\x0c\x12\x12\n\npoint_size\x18\x02 \x01(\x02\x1a\x97\x02\n\x07Overlay\x12!\n\x08position\x18\x01 \x01(\x0b\x32\x0f.Server.VecXYZf\x12\x0e\n\x04html\x18\x02 \x01(\tH\x00\x12\x30\n\x05image\x18\x03 \x01(\x0b\x32\x1f.Server.AddObject.Overlay.ImageH\x00\x12\x36\n\x04type\x18\x04 \x01(\x0e\x32(.Server.AddObject.Overlay.CoordinateType\x1a$\n\x05Image\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\x12\r\n\x05width\x18\x02 \x01(\x05\"=\n\x0e\x43oordinateType\x12\x14\n\x10WORLD_COORDINATE\x10\x00\x12\x15\n\x11SCREEN_COORDINATE\x10\x01\x42\n\n\x08\x43ontents\x1a\x90\x01\n\x04Mesh\x12\x1f\n\x06points\x18\x01 \x03(\x0b\x32\x0f.Server.VecXYZf\x12\x16\n\x0evertex_a_index\x18\x02 \x03(\x05\x12\x16\n\x0evertex_b_index\x18\x03 \x03(\x05\x12\x16\n\x0evertex_c_index\x18\x04 \x03(\x05\x12\x1f\n\x06\x63olors\x18\x05 \x03(\x0b\x32\x0f.Server.VecRGBf\x1a\x9a\x01\n\x05Image\x12\x13\n\x0b\x64ouble_side\x18\x01 \x01(\x08\x12#\n\nupper_left\x18\x02 \x01(\x0b\x32\x0f.Server.VecXYZf\x12#\n\nlower_left\x18\x03 \x01(\x0b\x32\x0f.Server.VecXYZf\x12$\n\x0blower_right\x18\x04 \x01(\x0b\x32\x0f.Server.VecXYZf\x12\x0c\n\x04\x64\x61ta\x18\x05 \x01(\x0c\x42\x08\n\x06Object\":\n\x0cRemoveObject\x12\r\n\x03\x61ll\x18\x01 \x01(\x08H\x00\x12\x11\n\x07\x62y_uuid\x18\x02 \x01(\tH\x00\x42\x08\n\x06Object\"U\n\x12SetKeyEventHandler\x12\x11\n\x07keydown\x18\x01 \x01(\x08H\x00\x12\x0f\n\x05keyup\x18\x02 \x01(\x08H\x00\x12\x12\n\x08keypress\x18\x03 \x01(\x08H\x00\x42\x07\n\x05\x45vent\"s\n\x1aSetCameraStateEventHandler\x12\x1b\n\x11\x61\x64\x64_with_interval\x18\x01 \x01(\x02H\x00\x12\x18\n\x0eremove_by_uuid\x18\x02 \x01(\tH\x00\x12\x14\n\nremove_all\x18\x03 \x01(\x08H\x00\x42\x08\n\x06\x41\x63tion\"*\n\x07VecXYZf\x12\t\n\x01x\x18\x01 \x01(\x02\x12\t\n\x01y\x18\x02 \x01(\x02\x12\t\n\x01z\x18\x03 \x01(\x02\"*\n\x07VecRGBf\x12\t\n\x01r\x18\x01 \x01(\x02\x12\t\n\x01g\x18\x02 \x01(\x02\x12\t\n\x01\x62\x18\x03 \x01(\x02\"\xdb\x06\n\x10SetCustomControl\x12\x0e\n\x06target\x18\x01 \x01(\t\x12\x31\n\x06slider\x18\x02 \x01(\x0b\x32\x1f.Server.SetCustomControl.SliderH\x00\x12\x35\n\x08\x63heckbox\x18\x03 \x01(\x0b\x32!.Server.SetCustomControl.CheckBoxH\x00\x12\x33\n\x07textbox\x18\x04 \x01(\x0b\x32 .Server.SetCustomControl.TextBoxH\x00\x12\x37\n\tselectbox\x18\x05 \x01(\x0b\x32\".Server.SetCustomControl.SelectBoxH\x00\x12\x31\n\x06\x62utton\x18\x06 \x01(\x0b\x32\x1f.Server.SetCustomControl.ButtonH\x00\x12<\n\x0c\x63olor_picker\x18\x07 \x01(\x0b\x32$.Server.SetCustomControl.ColorPickerH\x00\x1a\x92\x01\n\x06Slider\x12\x11\n\x04name\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x10\n\x03min\x18\x02 \x01(\x02H\x01\x88\x01\x01\x12\x10\n\x03max\x18\x03 \x01(\x02H\x02\x88\x01\x01\x12\x11\n\x04step\x18\x04 \x01(\x02H\x03\x88\x01\x01\x12\x12\n\x05value\x18\x05 \x01(\x02H\x04\x88\x01\x01\x42\x07\n\x05_nameB\x06\n\x04_minB\x06\n\x04_maxB\x07\n\x05_stepB\x08\n\x06_value\x1a\x44\n\x08\x43heckBox\x12\x11\n\x04name\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x12\n\x05value\x18\x02 \x01(\x08H\x01\x88\x01\x01\x42\x07\n\x05_nameB\x08\n\x06_value\x1a\x43\n\x07TextBox\x12\x11\n\x04name\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x12\n\x05value\x18\x02 \x01(\tH\x01\x88\x01\x01\x42\x07\n\x05_nameB\x08\n\x06_value\x1aT\n\tSelectBox\x12\x11\n\x04name\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\r\n\x05items\x18\x02 \x03(\t\x12\x12\n\x05value\x18\x03 \x01(\tH\x01\x88\x01\x01\x42\x07\n\x05_nameB\x08\n\x06_value\x1a$\n\x06\x42utton\x12\x11\n\x04name\x18\x01 \x01(\tH\x00\x88\x01\x01\x42\x07\n\x05_name\x1aG\n\x0b\x43olorPicker\x12\x11\n\x04name\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x12\n\x05value\x18\x02 \x01(\tH\x01\x88\x01\x01\x42\x07\n\x05_nameB\x08\n\x06_valueB\t\n\x07\x43ontrolb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'protobuf.server_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _globals['_SERVERCOMMAND']._serialized_start=34
  _globals['_SERVERCOMMAND']._serialized_end=612
  _globals['_CUSTOMCONTROL']._serialized_start=615
  _globals['_CUSTOMCONTROL']._serialized_end=1446
  _globals['_CUSTOMCONTROL_SLIDER']._serialized_start=991
  _globals['_CUSTOMCONTROL_SLIDER']._serialized_end=1089
  _globals['_CUSTOMCONTROL_CHECKBOX']._serialized_start=1091
  _globals['_CUSTOMCONTROL_CHECKBOX']._serialized_end=1151
  _globals['_CUSTOMCONTROL_TEXTBOX']._serialized_start=1153
  _globals['_CUSTOMCONTROL_TEXTBOX']._serialized_end=1212
  _globals['_CUSTOMCONTROL_SELECTBOX']._serialized_start=1214
  _globals['_CUSTOMCONTROL_SELECTBOX']._serialized_end=1290
  _globals['_CUSTOMCONTROL_BUTTON']._serialized_start=1292
  _globals['_CUSTOMCONTROL_BUTTON']._serialized_end=1330
  _globals['_CUSTOMCONTROL_COLORPICKER']._serialized_start=1332
  _globals['_CUSTOMCONTROL_COLORPICKER']._serialized_end=1395
  _globals['_CUSTOMCONTROL_FOLDER']._serialized_start=1397
  _globals['_CUSTOMCONTROL_FOLDER']._serialized_end=1435
  _globals['_REMOVECUSTOMCONTROL']._serialized_start=1448
  _globals['_REMOVECUSTOMCONTROL']._serialized_end=1513
  _globals['_SETCAMERA']._serialized_start=1516
  _globals['_SETCAMERA']._serialized_end=1883
  _globals['_SETCAMERA_ROLL']._serialized_start=1774
  _globals['_SETCAMERA_ROLL']._serialized_end=1824
  _globals['_SETCAMERA_CAMERAMODE']._serialized_start=1826
  _globals['_SETCAMERA_CAMERAMODE']._serialized_end=1873
  _globals['_ADDOBJECT']._serialized_start=1886
  _globals['_ADDOBJECT']._serialized_end=2905
  _globals['_ADDOBJECT_LINESET']._serialized_start=2128
  _globals['_ADDOBJECT_LINESET']._serialized_end=2257
  _globals['_ADDOBJECT_POINTCLOUD']._serialized_start=2259
  _globals['_ADDOBJECT_POINTCLOUD']._serialized_end=2309
  _globals['_ADDOBJECT_OVERLAY']._serialized_start=2312
  _globals['_ADDOBJECT_OVERLAY']._serialized_end=2591
  _globals['_ADDOBJECT_OVERLAY_IMAGE']._serialized_start=2480
  _globals['_ADDOBJECT_OVERLAY_IMAGE']._serialized_end=2516
  _globals['_ADDOBJECT_OVERLAY_COORDINATETYPE']._serialized_start=2518
  _globals['_ADDOBJECT_OVERLAY_COORDINATETYPE']._serialized_end=2579
  _globals['_ADDOBJECT_MESH']._serialized_start=2594
  _globals['_ADDOBJECT_MESH']._serialized_end=2738
  _globals['_ADDOBJECT_IMAGE']._serialized_start=2741
  _globals['_ADDOBJECT_IMAGE']._serialized_end=2895
  _globals['_REMOVEOBJECT']._serialized_start=2907
  _globals['_REMOVEOBJECT']._serialized_end=2965
  _globals['_SETKEYEVENTHANDLER']._serialized_start=2967
  _globals['_SETKEYEVENTHANDLER']._serialized_end=3052
  _globals['_SETCAMERASTATEEVENTHANDLER']._serialized_start=3054
  _globals['_SETCAMERASTATEEVENTHANDLER']._serialized_end=3169
  _globals['_VECXYZF']._serialized_start=3171
  _globals['_VECXYZF']._serialized_end=3213
  _globals['_VECRGBF']._serialized_start=3215
  _globals['_VECRGBF']._serialized_end=3257
  _globals['_SETCUSTOMCONTROL']._serialized_start=3260
  _globals['_SETCUSTOMCONTROL']._serialized_end=4119
  _globals['_SETCUSTOMCONTROL_SLIDER']._serialized_start=3626
  _globals['_SETCUSTOMCONTROL_SLIDER']._serialized_end=3772
  _globals['_SETCUSTOMCONTROL_CHECKBOX']._serialized_start=3774
  _globals['_SETCUSTOMCONTROL_CHECKBOX']._serialized_end=3842
  _globals['_SETCUSTOMCONTROL_TEXTBOX']._serialized_start=3844
  _globals['_SETCUSTOMCONTROL_TEXTBOX']._serialized_end=3911
  _globals['_SETCUSTOMCONTROL_SELECTBOX']._serialized_start=3913
  _globals['_SETCUSTOMCONTROL_SELECTBOX']._serialized_end=3997
  _globals['_SETCUSTOMCONTROL_BUTTON']._serialized_start=3999
  _globals['_SETCUSTOMCONTROL_BUTTON']._serialized_end=4035
  _globals['_SETCUSTOMCONTROL_COLORPICKER']._serialized_start=4037
  _globals['_SETCUSTOMCONTROL_COLORPICKER']._serialized_end=4108
# @@protoc_insertion_point(module_scope)



from enum import Enum
from dataclasses import dataclass
from cumo._internal.protobuf import client_pb2


class Vector3f:
    x: float
    y: float
    z: float

    def __init__(self, v: client_pb2.VecXYZf):
        self.x = v.x
        self.y = v.y
        self.z = v.z

    def __str__(self) -> str:
        return f"{self.__class__.__name__}({self.x}, {self.y}, {self.z})"


class CameraMode(Enum):
    PERSPECTIVE = 1
    ORTHOGRAPHIC = 2

    @staticmethod
    def _FromProtobuf(m: client_pb2.CameraState.CameraMode):
        if m == client_pb2.CameraState.CameraMode.PERSPECTIVE:
            return CameraMode.PERSPECTIVE
        return CameraMode.ORTHOGRAPHIC


@dataclass
class CameraState:
    position: Vector3f
    target: Vector3f
    up: Vector3f
    mode: CameraMode
    roll_lock: bool
    fov: float
    frustum_height: float

    def __str__(self) -> str:
        return f"{self.__class__.__name__}(position={self.position}, target={self.target}," \
            + f" up={self.up}, mode={self.mode}, roll_lock={self.roll_lock}," \
            + f" fov={self.fov}, frustum_height={self.frustum_height})"



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: protobuf/client.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x15protobuf/client.proto\x12\x06\x43lient\"\xcf\x02\n\rClientCommand\x12\x0c\n\x04UUID\x18\x01 \x01(\t\x12\x15\n\x0blog_message\x18\x02 \x01(\tH\x00\x12 \n\x06result\x18\x03 \x01(\x0b\x32\x0e.Client.ResultH\x00\x12\x1e\n\x05image\x18\x04 \x01(\x0b\x32\r.Client.ImageH\x00\x12\x31\n\x0f\x63ontrol_changed\x18\x05 \x01(\x0b\x32\x16.Client.ControlChangedH\x00\x12\x36\n\x12key_event_occurred\x18\x06 \x01(\x0b\x32\x18.Client.KeyEventOccurredH\x00\x12+\n\x0c\x63\x61mera_state\x18\x07 \x01(\x0b\x32\x13.Client.CameraStateH\x00\x12\x34\n\x15\x63\x61meara_state_changed\x18\x08 \x01(\x0b\x32\x13.Client.CameraStateH\x00\x42\t\n\x07\x43ommand\"8\n\x06Result\x12\x11\n\x07success\x18\x01 \x01(\tH\x00\x12\x11\n\x07\x66\x61ilure\x18\x02 \x01(\tH\x00\x42\x08\n\x06Result\"\x15\n\x05Image\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\"N\n\x0e\x43ontrolChanged\x12\x10\n\x06number\x18\x01 \x01(\x01H\x00\x12\x0e\n\x04text\x18\x02 \x01(\tH\x00\x12\x11\n\x07\x62oolean\x18\x03 \x01(\x08H\x00\x42\x07\n\x05Value\"\xb7\x02\n\x10KeyEventOccurred\x12\x32\n\x05keyup\x18\x01 \x01(\x0b\x32!.Client.KeyEventOccurred.KeyEventH\x00\x12\x34\n\x07keydown\x18\x02 \x01(\x0b\x32!.Client.KeyEventOccurred.KeyEventH\x00\x12\x35\n\x08keypress\x18\x03 \x01(\x0b\x32!.Client.KeyEventOccurred.KeyEventH\x00\x1ay\n\x08KeyEvent\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\x12\x10\n\x08shiftKey\x18\x03 \x01(\x08\x12\x0e\n\x06\x61ltKey\x18\x04 \x01(\x08\x12\x0f\n\x07\x63trlKey\x18\x05 \x01(\x08\x12\x0f\n\x07metaKey\x18\x06 \x01(\x08\x12\x0e\n\x06repeat\x18\x07 \x01(\x08\x42\x07\n\x05\x45vent\"\x85\x02\n\x0b\x43\x61meraState\x12!\n\x08position\x18\x01 \x01(\x0b\x32\x0f.Client.VecXYZf\x12\x1f\n\x06target\x18\x02 \x01(\x0b\x32\x0f.Client.VecXYZf\x12\x1b\n\x02up\x18\x03 \x01(\x0b\x32\x0f.Client.VecXYZf\x12,\n\x04mode\x18\x04 \x01(\x0e\x32\x1e.Client.CameraState.CameraMode\x12\x11\n\troll_lock\x18\x05 \x01(\x08\x12\x0b\n\x03\x66ov\x18\x06 \x01(\x02\x12\x16\n\x0e\x66rustum_height\x18\x07 \x01(\x02\"/\n\nCameraMode\x12\x0f\n\x0bPERSPECTIVE\x10\x00\x12\x10\n\x0cORTHOGRAPHIC\x10\x01\"*\n\x07VecXYZf\x12\t\n\x01x\x18\x01 \x01(\x02\x12\t\n\x01y\x18\x02 \x01(\x02\x12\t\n\x01z\x18\x03 \x01(\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'protobuf.client_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _globals['_CLIENTCOMMAND']._serialized_start=34
  _globals['_CLIENTCOMMAND']._serialized_end=369
  _globals['_RESULT']._serialized_start=371
  _globals['_RESULT']._serialized_end=427
  _globals['_IMAGE']._serialized_start=429
  _globals['_IMAGE']._serialized_end=450
  _globals['_CONTROLCHANGED']._serialized_start=452
  _globals['_CONTROLCHANGED']._serialized_end=530
  _globals['_KEYEVENTOCCURRED']._serialized_start=533
  _globals['_KEYEVENTOCCURRED']._serialized_end=844
  _globals['_KEYEVENTOCCURRED_KEYEVENT']._serialized_start=714
  _globals['_KEYEVENTOCCURRED_KEYEVENT']._serialized_end=835
  _globals['_CAMERASTATE']._serialized_start=847
  _globals['_CAMERASTATE']._serialized_end=1108
  _globals['_CAMERASTATE_CAMERAMODE']._serialized_start=1061
  _globals['_CAMERASTATE_CAMERAMODE']._serialized_end=1108
  _globals['_VECXYZF']._serialized_start=1110
  _globals['_VECXYZF']._serialized_end=1152
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class ClientCommand(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    UUID_FIELD_NUMBER: builtins.int
    LOG_MESSAGE_FIELD_NUMBER: builtins.int
    RESULT_FIELD_NUMBER: builtins.int
    IMAGE_FIELD_NUMBER: builtins.int
    CONTROL_CHANGED_FIELD_NUMBER: builtins.int
    KEY_EVENT_OCCURRED_FIELD_NUMBER: builtins.int
    CAMERA_STATE_FIELD_NUMBER: builtins.int
    CAMEARA_STATE_CHANGED_FIELD_NUMBER: builtins.int
    UUID: builtins.str
    log_message: builtins.str
    @property
    def result(self) -> global___Result: ...
    @property
    def image(self) -> global___Image: ...
    @property
    def control_changed(self) -> global___ControlChanged: ...
    @property
    def key_event_occurred(self) -> global___KeyEventOccurred: ...
    @property
    def camera_state(self) -> global___CameraState: ...
    @property
    def cameara_state_changed(self) -> global___CameraState: ...
    def __init__(
        self,
        *,
        UUID: builtins.str = ...,
        log_message: builtins.str = ...,
        result: global___Result | None = ...,
        image: global___Image | None = ...,
        control_changed: global___ControlChanged | None = ...,
        key_event_occurred: global___KeyEventOccurred | None = ...,
        camera_state: global___CameraState | None = ...,
        cameara_state_changed: global___CameraState | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["Command", b"Command", "cameara_state_changed", b"cameara_state_changed", "camera_state", b"camera_state", "control_changed", b"control_changed", "image", b"image", "key_event_occurred", b"key_event_occurred", "log_message", b"log_message", "result", b"result"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["Command", b"Command", "UUID", b"UUID", "cameara_state_changed", b"cameara_state_changed", "camera_state", b"camera_state", "control_changed", b"control_changed", "image", b"image", "key_event_occurred", b"key_event_occurred", "log_message", b"log_message", "result", b"result"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["Command", b"Command"]) -> typing_extensions.Literal["log_message", "result", "image", "control_changed", "key_event_occurred", "camera_state", "cameara_state_changed"] | None: ...

global___ClientCommand = ClientCommand

@typing_extensions.final
class Result(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SUCCESS_FIELD_NUMBER: builtins.int
    FAILURE_FIELD_NUMBER: builtins.int
    success: builtins.str
    failure: builtins.str
    def __init__(
        self,
        *,
        success: builtins.str = ...,
        failure: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["Result", b"Result", "failure", b"failure", "success", b"success"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["Result", b"Result", "failure", b"failure", "success", b"success"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["Result", b"Result"]) -> typing_extensions.Literal["success", "failure"] | None: ...

global___Result = Result

@typing_extensions.final
class Image(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATA_FIELD_NUMBER: builtins.int
    data: builtins.bytes
    def __init__(
        self,
        *,
        data: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["data", b"data"]) -> None: ...

global___Image = Image

@typing_extensions.final
class ControlChanged(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NUMBER_FIELD_NUMBER: builtins.int
    TEXT_FIELD_NUMBER: builtins.int
    BOOLEAN_FIELD_NUMBER: builtins.int
    number: builtins.float
    text: builtins.str
    boolean: builtins.bool
    def __init__(
        self,
        *,
        number: builtins.float = ...,
        text: builtins.str = ...,
        boolean: builtins.bool = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["Value", b"Value", "boolean", b"boolean", "number", b"number", "text", b"text"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["Value", b"Value", "boolean", b"boolean", "number", b"number", "text", b"text"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["Value", b"Value"]) -> typing_extensions.Literal["number", "text", "boolean"] | None: ...

global___ControlChanged = ControlChanged

@typing_extensions.final
class KeyEventOccurred(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing_extensions.final
    class KeyEvent(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        CODE_FIELD_NUMBER: builtins.int
        SHIFTKEY_FIELD_NUMBER: builtins.int
        ALTKEY_FIELD_NUMBER: builtins.int
        CTRLKEY_FIELD_NUMBER: builtins.int
        METAKEY_FIELD_NUMBER: builtins.int
        REPEAT_FIELD_NUMBER: builtins.int
        key: builtins.str
        code: builtins.str
        shiftKey: builtins.bool
        altKey: builtins.bool
        ctrlKey: builtins.bool
        metaKey: builtins.bool
        repeat: builtins.bool
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            code: builtins.str = ...,
            shiftKey: builtins.bool = ...,
            altKey: builtins.bool = ...,
            ctrlKey: builtins.bool = ...,
            metaKey: builtins.bool = ...,
            repeat: builtins.bool = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["altKey", b"altKey", "code", b"code", "ctrlKey", b"ctrlKey", "key", b"key", "metaKey", b"metaKey", "repeat", b"repeat", "shiftKey", b"shiftKey"]) -> None: ...

    KEYUP_FIELD_NUMBER: builtins.int
    KEYDOWN_FIELD_NUMBER: builtins.int
    KEYPRESS_FIELD_NUMBER: builtins.int
    @property
    def keyup(self) -> global___KeyEventOccurred.KeyEvent: ...
    @property
    def keydown(self) -> global___KeyEventOccurred.KeyEvent: ...
    @property
    def keypress(self) -> global___KeyEventOccurred.KeyEvent: ...
    def __init__(
        self,
        *,
        keyup: global___KeyEventOccurred.KeyEvent | None = ...,
        keydown: global___KeyEventOccurred.KeyEvent | None = ...,
        keypress: global___KeyEventOccurred.KeyEvent | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["Event", b"Event", "keydown", b"keydown", "keypress", b"keypress", "keyup", b"keyup"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["Event", b"Event", "keydown", b"keydown", "keypress", b"keypress", "keyup", b"keyup"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["Event", b"Event"]) -> typing_extensions.Literal["keyup", "keydown", "keypress"] | None: ...

global___KeyEventOccurred = KeyEventOccurred

@typing_extensions.final
class CameraState(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _CameraMode:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _CameraModeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[CameraState._CameraMode.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        PERSPECTIVE: CameraState._CameraMode.ValueType  # 0
        ORTHOGRAPHIC: CameraState._CameraMode.ValueType  # 1

    class CameraMode(_CameraMode, metaclass=_CameraModeEnumTypeWrapper): ...
    PERSPECTIVE: CameraState.CameraMode.ValueType  # 0
    ORTHOGRAPHIC: CameraState.CameraMode.ValueType  # 1

    POSITION_FIELD_NUMBER: builtins.int
    TARGET_FIELD_NUMBER: builtins.int
    UP_FIELD_NUMBER: builtins.int
    MODE_FIELD_NUMBER: builtins.int
    ROLL_LOCK_FIELD_NUMBER: builtins.int
    FOV_FIELD_NUMBER: builtins.int
    FRUSTUM_HEIGHT_FIELD_NUMBER: builtins.int
    @property
    def position(self) -> global___VecXYZf: ...
    @property
    def target(self) -> global___VecXYZf: ...
    @property
    def up(self) -> global___VecXYZf: ...
    mode: global___CameraState.CameraMode.ValueType
    roll_lock: builtins.bool
    fov: builtins.float
    frustum_height: builtins.float
    def __init__(
        self,
        *,
        position: global___VecXYZf | None = ...,
        target: global___VecXYZf | None = ...,
        up: global___VecXYZf | None = ...,
        mode: global___CameraState.CameraMode.ValueType = ...,
        roll_lock: builtins.bool = ...,
        fov: builtins.float = ...,
        frustum_height: builtins.float = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["position", b"position", "target", b"target", "up", b"up"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["fov", b"fov", "frustum_height", b"frustum_height", "mode", b"mode", "position", b"position", "roll_lock", b"roll_lock", "target", b"target", "up", b"up"]) -> None: ...

global___CameraState = CameraState

@typing_extensions.final
class VecXYZf(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    X_FIELD_NUMBER: builtins.int
    Y_FIELD_NUMBER: builtins.int
    Z_FIELD_NUMBER: builtins.int
    x: builtins.float
    y: builtins.float
    z: builtins.float
    def __init__(
        self,
        *,
        x: builtins.float = ...,
        y: builtins.float = ...,
        z: builtins.float = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["x", b"x", "y", b"y", "z", b"z"]) -> None: ...

global___VecXYZf = VecXYZf



from __future__ import annotations  # Postponed Evaluation of Annotations
from uuid import UUID, uuid4
from math import sqrt, isfinite
from typing import TYPE_CHECKING, Optional, Callable
from cumo._internal.protobuf import server_pb2
from cumo.camera_state import CameraState, Vector3f, CameraMode
if TYPE_CHECKING:
    from cumo import PointCloudViewer
# pylint: disable=no-member

_EVENT_CAMERA_STATE_CHANGED = "camerastatechanged"


def set_orthographic_camera(
    self: PointCloudViewer,
    frustum_height: Optional[float] = None,
) -> None:
    """カメラを正投影カメラに切り替えさせる。

    :param frustum_height: カメラの視錐台の高さ。幅はウィンドウのアスペクト比から計算される
    :type frustum_height: float, optional
    """
    camera = server_pb2.SetCamera()
    if frustum_height is None:
        camera.mode = server_pb2.SetCamera.CameraMode.ORTHOGRAPHIC
    else:
        camera.orthographic_frustum_height = frustum_height
    obj = server_pb2.ServerCommand()
    obj.set_camera.CopyFrom(camera)
    uuid = uuid4()
    self._send_data(obj, uuid)
    ret = self._wait_until(uuid)
    if ret.result.HasField("failure"):
        raise RuntimeError(ret.result.failure)


def set_perspective_camera(
    self: PointCloudViewer,
    fov: Optional[float] = None,
) -> None:
    """カメラを遠近投影カメラに切り替えさせる。

    :param fov: 視野角
    :type fov: float, optional
    """
    camera = server_pb2.SetCamera()
    if fov is None:
        camera.mode = server_pb2.SetCamera.CameraMode.PERSPECTIVE
    else:
        camera.perspective_fov = fov
    obj = server_pb2.ServerCommand()
    obj.set_camera.CopyFrom(camera)
    uuid = uuid4()
    self._send_data(obj, uuid)
    ret = self._wait_until(uuid)
    if ret.result.HasField("failure"):
        raise RuntimeError(ret.result.failure)


def set_camera_position(
    self: PointCloudViewer,
    x: float,
    y: float,
    z: float,
) -> None:
    """カメラの位置を変更する。

    :param x: カメラのx座標
    :type x: float
    :param y: カメラのy座標
    :type y: float
    :param z: カメラのz座標
    :type z: float
    """
    position = server_pb2.VecXYZf()
    position.x = x
    position.y = y
    position.z = z

    camera = server_pb2.SetCamera()
    camera.position.CopyFrom(position)

    obj = server_pb2.ServerCommand()
    obj.set_camera.CopyFrom(camera)
    uuid = uuid4()
    self._send_data(obj, uuid)
    ret = self._wait_until(uuid)
    if ret.result.HasField("failure"):
        raise RuntimeError(ret.result.failure)


def set_camera_target(
    self: PointCloudViewer,
    x: float,
    y: float,
    z: float,
) -> None:
    """カメラが向く目標の座標を変更する。

    :param x: 目標のx座標
    :type x: float
    :param y: 目標のy座標
    :type y: float
    :param z: 目標のz座標
    :type z: float
    """
    target = server_pb2.VecXYZf()
    target.x = x
    target.y = y
    target.z = z

    camera = server_pb2.SetCamera()
    camera.target.CopyFrom(target)

    obj = server_pb2.ServerCommand()
    obj.set_camera.CopyFrom(camera)
    uuid = uuid4()
    self._send_data(obj, uuid)
    ret = self._wait_until(uuid)
    if ret.result.HasField("failure"):
        raise RuntimeError(ret.result.failure)


def set_camera_roll(
    self: PointCloudViewer,
    angle_rad: float,
    up_x: float = 0,
    up_y: float = 1,
    up_z: float = 0,
) -> None:
    """カメラのロール(視線を軸とした回転角度)を設定する。

    Args:
        roll_rad (float): 回転角度。ラジアンで指定する
        up_x (float): 回転角度の基準となるベクトルのx成分。この方向が画面の上を指す状態が0度になる
        up_y (float): 回転角度の基準となるベクトルのy成分。この方向が画面の上を指す状態が0度になる
        up_z (float): 回転角度の基準となるベクトルのz成分。この方向が画面の上を指す状態が0度になる
    """

    norm_sq = up_x*up_x + up_y*up_y + up_z*up_z
    if norm_sq == 0:
        raise ValueError("up must not be zero")

    norm = sqrt(norm_sq)

    up = server_pb2.VecXYZf()
    up.x = up_x / norm
    up.y = up_y / norm
    up.z = up_z / norm

    roll = server_pb2.SetCamera.Roll()
    roll.angle = angle_rad
    roll.up.CopyFrom(up)

    camera = server_pb2.SetCamera()
    camera.roll.CopyFrom(roll)

    obj = server_pb2.ServerCommand()
    obj.set_camera.CopyFrom(camera)
    uuid = uuid4()
    self._send_data(obj, uuid)
    ret = self._wait_until(uuid)
    if ret.result.HasField("failure"):
        raise RuntimeError(ret.result.failure)


def set_camera_roll_lock(
    self: PointCloudViewer,
    enable: bool,
):
    """カメラのロールを固定し、画面の上側が常に同じ方向を向くようにする機能を有効化、または無効化する。

    Args:
        enable (bool): Trueにセットすることでカメラの"上"が固定され、それが変更されるようなマウス操作が無効化される。
    """
    camera = server_pb2.SetCamera()
    camera.roll_lock = enable

    obj = server_pb2.ServerCommand()
    obj.set_camera.CopyFrom(camera)
    uuid = uuid4()
    self._send_data(obj, uuid)
    ret = self._wait_until(uuid)
    if ret.result.HasField("failure"):
        raise RuntimeError(ret.result.failure)


def get_camera_state(
    self: PointCloudViewer
) -> CameraState:
    """カメラの状態を取得する

    Args:
        self (PointCloudViewer): _description_

    Raises:
        RuntimeError: _description_

    Returns:
        _type_: _description_
    """
    obj = server_pb2.ServerCommand()
    obj.get_camera_state = True
    uuid = uuid4()
    self._send_data(obj, uuid)
    ret = self._wait_until(uuid)
    if ret.HasField("camera_state"):
        return CameraState(
            position=Vector3f(ret.camera_state.position),
            target=Vector3f(ret.camera_state.target),
            up=Vector3f(ret.camera_state.up),
            mode=CameraMode._FromProtobuf(ret.camera_state.mode),
            roll_lock=ret.camera_state.roll_lock,
            fov=ret.camera_state.fov,
            frustum_height=ret.camera_state.frustum_height
        )
    if ret.result.HasField("failure"):
        raise RuntimeError(ret.result.failure)
    raise RuntimeError("unexpected response")


def add_camera_state_changed_handler(
        self: PointCloudViewer,
        handler: Callable[[CameraState, UUID], None],
        interval: float = 0.1
) -> UUID:
    """ブラウザでカメラの状態変化が発生したときに呼ばれるハンドラーを登録する。

    Args:
        handler (Callable[[CameraState, UUID], None]): ブラウザでカメラの状態変化が発生したときに呼ばれるハンドラー
        interval float: イベントの最小発生間隔(秒)。

    Returns:
        UUID: ハンドラーに対応するID。後から操作する際に使う
    """
    if interval < 0 or not isfinite(interval):
        raise ValueError("interval must be finite number zero or greater")
    obj = server_pb2.ServerCommand()
    c = server_pb2.SetCameraStateEventHandler(
        add_with_interval=interval
    )
    obj.set_camera_state_event_handler.CopyFrom(c)
    uuid = uuid4()
    self._send_data(obj, uuid)
    ret = self._wait_until(uuid)
    if ret.result.HasField("failure"):
        raise RuntimeError(ret.result.failure)

    self._set_custom_handler(uuid, _EVENT_CAMERA_STATE_CHANGED, handler)
    return uuid


def remove_camera_state_changed_handler(
    self: PointCloudViewer,
    uuid: Optional[UUID] = None,
) -> None:
    """ブラウザでカメラの状態変化が発生したときに呼ばれるハンドラーを削除する。

    Args:
        uuid (Optional[UUID], optional): 削除するハンドラのUUID、指定しない場合すべて削除する
    """
    if uuid is not None:
        if self._get_custom_handler(uuid, _EVENT_CAMERA_STATE_CHANGED) is None:
            raise KeyError(uuid)
        self._custom_handlers[_EVENT_CAMERA_STATE_CHANGED].pop(uuid)
    else:
        if _EVENT_CAMERA_STATE_CHANGED in self._custom_handlers:
            self._custom_handlers[_EVENT_CAMERA_STATE_CHANGED].clear()
    obj = server_pb2.ServerCommand()
    r = server_pb2.SetCameraStateEventHandler()
    if uuid is not None:
        r.remove_by_uuid = str(uuid)
    else:
        r.remove_all = True
    obj.set_camera_state_event_handler.CopyFrom(r)
    uuid = uuid4()
    self._send_data(obj, uuid)
    ret = self._wait_until(uuid)
    if ret.result.HasField("failure"):
        raise RuntimeError(ret.result.failure)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class ServerCommand(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    UUID_FIELD_NUMBER: builtins.int
    LOG_MESSAGE_FIELD_NUMBER: builtins.int
    CAPTURE_SCREEN_FIELD_NUMBER: builtins.int
    ADD_CUSTOM_CONTROL_FIELD_NUMBER: builtins.int
    SET_CAMERA_FIELD_NUMBER: builtins.int
    ADD_OBJECT_FIELD_NUMBER: builtins.int
    SET_KEY_EVENT_HANDLER_FIELD_NUMBER: builtins.int
    REMOVE_OBJECT_FIELD_NUMBER: builtins.int
    REMOVE_CUSTOM_CONTROL_FIELD_NUMBER: builtins.int
    SET_CUSTOM_CONTROL_FIELD_NUMBER: builtins.int
    SET_ENABLE_FIELD_NUMBER: builtins.int
    GET_CAMERA_STATE_FIELD_NUMBER: builtins.int
    SET_CAMERA_STATE_EVENT_HANDLER_FIELD_NUMBER: builtins.int
    UUID: builtins.str
    log_message: builtins.str
    capture_screen: builtins.bool
    @property
    def add_custom_control(self) -> global___CustomControl: ...
    @property
    def set_camera(self) -> global___SetCamera: ...
    @property
    def add_object(self) -> global___AddObject: ...
    @property
    def set_key_event_handler(self) -> global___SetKeyEventHandler: ...
    @property
    def remove_object(self) -> global___RemoveObject: ...
    @property
    def remove_custom_control(self) -> global___RemoveCustomControl: ...
    @property
    def set_custom_control(self) -> global___SetCustomControl: ...
    set_enable: builtins.bool
    get_camera_state: builtins.bool
    @property
    def set_camera_state_event_handler(self) -> global___SetCameraStateEventHandler: ...
    def __init__(
        self,
        *,
        UUID: builtins.str = ...,
        log_message: builtins.str = ...,
        capture_screen: builtins.bool = ...,
        add_custom_control: global___CustomControl | None = ...,
        set_camera: global___SetCamera | None = ...,
        add_object: global___AddObject | None = ...,
        set_key_event_handler: global___SetKeyEventHandler | None = ...,
        remove_object: global___RemoveObject | None = ...,
        remove_custom_control: global___RemoveCustomControl | None = ...,
        set_custom_control: global___SetCustomControl | None = ...,
        set_enable: builtins.bool = ...,
        get_camera_state: builtins.bool = ...,
        set_camera_state_event_handler: global___SetCameraStateEventHandler | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["Command", b"Command", "add_custom_control", b"add_custom_control", "add_object", b"add_object", "capture_screen", b"capture_screen", "get_camera_state", b"get_camera_state", "log_message", b"log_message", "remove_custom_control", b"remove_custom_control", "remove_object", b"remove_object", "set_camera", b"set_camera", "set_camera_state_event_handler", b"set_camera_state_event_handler", "set_custom_control", b"set_custom_control", "set_enable", b"set_enable", "set_key_event_handler", b"set_key_event_handler"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["Command", b"Command", "UUID", b"UUID", "add_custom_control", b"add_custom_control", "add_object", b"add_object", "capture_screen", b"capture_screen", "get_camera_state", b"get_camera_state", "log_message", b"log_message", "remove_custom_control", b"remove_custom_control", "remove_object", b"remove_object", "set_camera", b"set_camera", "set_camera_state_event_handler", b"set_camera_state_event_handler", "set_custom_control", b"set_custom_control", "set_enable", b"set_enable", "set_key_event_handler", b"set_key_event_handler"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["Command", b"Command"]) -> typing_extensions.Literal["log_message", "capture_screen", "add_custom_control", "set_camera", "add_object", "set_key_event_handler", "remove_object", "remove_custom_control", "set_custom_control", "set_enable", "get_camera_state", "set_camera_state_event_handler"] | None: ...

global___ServerCommand = ServerCommand

@typing_extensions.final
class CustomControl(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing_extensions.final
    class Slider(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        NAME_FIELD_NUMBER: builtins.int
        MIN_FIELD_NUMBER: builtins.int
        MAX_FIELD_NUMBER: builtins.int
        STEP_FIELD_NUMBER: builtins.int
        INIT_VALUE_FIELD_NUMBER: builtins.int
        PARENT_FIELD_NUMBER: builtins.int
        name: builtins.str
        min: builtins.float
        max: builtins.float
        step: builtins.float
        init_value: builtins.float
        parent: builtins.str
        def __init__(
            self,
            *,
            name: builtins.str = ...,
            min: builtins.float = ...,
            max: builtins.float = ...,
            step: builtins.float = ...,
            init_value: builtins.float = ...,
            parent: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["init_value", b"init_value", "max", b"max", "min", b"min", "name", b"name", "parent", b"parent", "step", b"step"]) -> None: ...

    @typing_extensions.final
    class CheckBox(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        NAME_FIELD_NUMBER: builtins.int
        INIT_VALUE_FIELD_NUMBER: builtins.int
        PARENT_FIELD_NUMBER: builtins.int
        name: builtins.str
        init_value: builtins.bool
        parent: builtins.str
        def __init__(
            self,
            *,
            name: builtins.str = ...,
            init_value: builtins.bool = ...,
            parent: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["init_value", b"init_value", "name", b"name", "parent", b"parent"]) -> None: ...

    @typing_extensions.final
    class TextBox(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        NAME_FIELD_NUMBER: builtins.int
        INIT_VALUE_FIELD_NUMBER: builtins.int
        PARENT_FIELD_NUMBER: builtins.int
        name: builtins.str
        init_value: builtins.str
        parent: builtins.str
        def __init__(
            self,
            *,
            name: builtins.str = ...,
            init_value: builtins.str = ...,
            parent: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["init_value", b"init_value", "name", b"name", "parent", b"parent"]) -> None: ...

    @typing_extensions.final
    class SelectBox(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        NAME_FIELD_NUMBER: builtins.int
        ITEMS_FIELD_NUMBER: builtins.int
        INIT_VALUE_FIELD_NUMBER: builtins.int
        PARENT_FIELD_NUMBER: builtins.int
        name: builtins.str
        @property
        def items(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
        init_value: builtins.str
        parent: builtins.str
        def __init__(
            self,
            *,
            name: builtins.str = ...,
            items: collections.abc.Iterable[builtins.str] | None = ...,
            init_value: builtins.str = ...,
            parent: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["init_value", b"init_value", "items", b"items", "name", b"name", "parent", b"parent"]) -> None: ...

    @typing_extensions.final
    class Button(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        NAME_FIELD_NUMBER: builtins.int
        PARENT_FIELD_NUMBER: builtins.int
        name: builtins.str
        parent: builtins.str
        def __init__(
            self,
            *,
            name: builtins.str = ...,
            parent: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["name", b"name", "parent", b"parent"]) -> None: ...

    @typing_extensions.final
    class ColorPicker(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        NAME_FIELD_NUMBER: builtins.int
        INIT_VALUE_FIELD_NUMBER: builtins.int
        PARENT_FIELD_NUMBER: builtins.int
        name: builtins.str
        init_value: builtins.str
        parent: builtins.str
        def __init__(
            self,
            *,
            name: builtins.str = ...,
            init_value: builtins.str = ...,
            parent: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["init_value", b"init_value", "name", b"name", "parent", b"parent"]) -> None: ...

    @typing_extensions.final
    class Folder(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        NAME_FIELD_NUMBER: builtins.int
        PARENT_FIELD_NUMBER: builtins.int
        name: builtins.str
        parent: builtins.str
        def __init__(
            self,
            *,
            name: builtins.str = ...,
            parent: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["name", b"name", "parent", b"parent"]) -> None: ...

    SLIDER_FIELD_NUMBER: builtins.int
    CHECKBOX_FIELD_NUMBER: builtins.int
    TEXTBOX_FIELD_NUMBER: builtins.int
    SELECTBOX_FIELD_NUMBER: builtins.int
    BUTTON_FIELD_NUMBER: builtins.int
    COLOR_PICKER_FIELD_NUMBER: builtins.int
    FOLDER_FIELD_NUMBER: builtins.int
    @property
    def slider(self) -> global___CustomControl.Slider: ...
    @property
    def checkbox(self) -> global___CustomControl.CheckBox: ...
    @property
    def textbox(self) -> global___CustomControl.TextBox: ...
    @property
    def selectbox(self) -> global___CustomControl.SelectBox: ...
    @property
    def button(self) -> global___CustomControl.Button: ...
    @property
    def color_picker(self) -> global___CustomControl.ColorPicker: ...
    @property
    def folder(self) -> global___CustomControl.Folder: ...
    def __init__(
        self,
        *,
        slider: global___CustomControl.Slider | None = ...,
        checkbox: global___CustomControl.CheckBox | None = ...,
        textbox: global___CustomControl.TextBox | None = ...,
        selectbox: global___CustomControl.SelectBox | None = ...,
        button: global___CustomControl.Button | None = ...,
        color_picker: global___CustomControl.ColorPicker | None = ...,
        folder: global___CustomControl.Folder | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["Control", b"Control", "button", b"button", "checkbox", b"checkbox", "color_picker", b"color_picker", "folder", b"folder", "selectbox", b"selectbox", "slider", b"slider", "textbox", b"textbox"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["Control", b"Control", "button", b"button", "checkbox", b"checkbox", "color_picker", b"color_picker", "folder", b"folder", "selectbox", b"selectbox", "slider", b"slider", "textbox", b"textbox"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["Control", b"Control"]) -> typing_extensions.Literal["slider", "checkbox", "textbox", "selectbox", "button", "color_picker", "folder"] | None: ...

global___CustomControl = CustomControl

@typing_extensions.final
class RemoveCustomControl(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ALL_FIELD_NUMBER: builtins.int
    BY_UUID_FIELD_NUMBER: builtins.int
    all: builtins.bool
    by_uuid: builtins.str
    def __init__(
        self,
        *,
        all: builtins.bool = ...,
        by_uuid: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["Object", b"Object", "all", b"all", "by_uuid", b"by_uuid"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["Object", b"Object", "all", b"all", "by_uuid", b"by_uuid"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["Object", b"Object"]) -> typing_extensions.Literal["all", "by_uuid"] | None: ...

global___RemoveCustomControl = RemoveCustomControl

@typing_extensions.final
class SetCamera(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _CameraMode:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _CameraModeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[SetCamera._CameraMode.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        PERSPECTIVE: SetCamera._CameraMode.ValueType  # 0
        ORTHOGRAPHIC: SetCamera._CameraMode.ValueType  # 1

    class CameraMode(_CameraMode, metaclass=_CameraModeEnumTypeWrapper): ...
    PERSPECTIVE: SetCamera.CameraMode.ValueType  # 0
    ORTHOGRAPHIC: SetCamera.CameraMode.ValueType  # 1

    @typing_extensions.final
    class Roll(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        ANGLE_FIELD_NUMBER: builtins.int
        UP_FIELD_NUMBER: builtins.int
        angle: builtins.float
        @property
        def up(self) -> global___VecXYZf: ...
        def __init__(
            self,
            *,
            angle: builtins.float = ...,
            up: global___VecXYZf | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["up", b"up"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["angle", b"angle", "up", b"up"]) -> None: ...

    PERSPECTIVE_FOV_FIELD_NUMBER: builtins.int
    ORTHOGRAPHIC_FRUSTUM_HEIGHT_FIELD_NUMBER: builtins.int
    POSITION_FIELD_NUMBER: builtins.int
    TARGET_FIELD_NUMBER: builtins.int
    ROLL_FIELD_NUMBER: builtins.int
    ROLL_LOCK_FIELD_NUMBER: builtins.int
    MODE_FIELD_NUMBER: builtins.int
    perspective_fov: builtins.float
    orthographic_frustum_height: builtins.float
    @property
    def position(self) -> global___VecXYZf: ...
    @property
    def target(self) -> global___VecXYZf: ...
    @property
    def roll(self) -> global___SetCamera.Roll: ...
    roll_lock: builtins.bool
    mode: global___SetCamera.CameraMode.ValueType
    def __init__(
        self,
        *,
        perspective_fov: builtins.float = ...,
        orthographic_frustum_height: builtins.float = ...,
        position: global___VecXYZf | None = ...,
        target: global___VecXYZf | None = ...,
        roll: global___SetCamera.Roll | None = ...,
        roll_lock: builtins.bool = ...,
        mode: global___SetCamera.CameraMode.ValueType = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["Camera", b"Camera", "mode", b"mode", "orthographic_frustum_height", b"orthographic_frustum_height", "perspective_fov", b"perspective_fov", "position", b"position", "roll", b"roll", "roll_lock", b"roll_lock", "target", b"target"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["Camera", b"Camera", "mode", b"mode", "orthographic_frustum_height", b"orthographic_frustum_height", "perspective_fov", b"perspective_fov", "position", b"position", "roll", b"roll", "roll_lock", b"roll_lock", "target", b"target"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["Camera", b"Camera"]) -> typing_extensions.Literal["perspective_fov", "orthographic_frustum_height", "position", "target", "roll", "roll_lock", "mode"] | None: ...

global___SetCamera = SetCamera

@typing_extensions.final
class AddObject(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing_extensions.final
    class LineSet(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        POINTS_FIELD_NUMBER: builtins.int
        FROM_INDEX_FIELD_NUMBER: builtins.int
        TO_INDEX_FIELD_NUMBER: builtins.int
        COLORS_FIELD_NUMBER: builtins.int
        WIDTHS_FIELD_NUMBER: builtins.int
        @property
        def points(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___VecXYZf]: ...
        @property
        def from_index(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
        @property
        def to_index(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
        @property
        def colors(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___VecRGBf]: ...
        @property
        def widths(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
        def __init__(
            self,
            *,
            points: collections.abc.Iterable[global___VecXYZf] | None = ...,
            from_index: collections.abc.Iterable[builtins.int] | None = ...,
            to_index: collections.abc.Iterable[builtins.int] | None = ...,
            colors: collections.abc.Iterable[global___VecRGBf] | None = ...,
            widths: collections.abc.Iterable[builtins.float] | None = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["colors", b"colors", "from_index", b"from_index", "points", b"points", "to_index", b"to_index", "widths", b"widths"]) -> None: ...

    @typing_extensions.final
    class PointCloud(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        PCD_DATA_FIELD_NUMBER: builtins.int
        POINT_SIZE_FIELD_NUMBER: builtins.int
        pcd_data: builtins.bytes
        point_size: builtins.float
        def __init__(
            self,
            *,
            pcd_data: builtins.bytes = ...,
            point_size: builtins.float = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["pcd_data", b"pcd_data", "point_size", b"point_size"]) -> None: ...

    @typing_extensions.final
    class Overlay(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        class _CoordinateType:
            ValueType = typing.NewType("ValueType", builtins.int)
            V: typing_extensions.TypeAlias = ValueType

        class _CoordinateTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[AddObject.Overlay._CoordinateType.ValueType], builtins.type):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
            WORLD_COORDINATE: AddObject.Overlay._CoordinateType.ValueType  # 0
            SCREEN_COORDINATE: AddObject.Overlay._CoordinateType.ValueType  # 1

        class CoordinateType(_CoordinateType, metaclass=_CoordinateTypeEnumTypeWrapper): ...
        WORLD_COORDINATE: AddObject.Overlay.CoordinateType.ValueType  # 0
        SCREEN_COORDINATE: AddObject.Overlay.CoordinateType.ValueType  # 1

        @typing_extensions.final
        class Image(google.protobuf.message.Message):
            DESCRIPTOR: google.protobuf.descriptor.Descriptor

            DATA_FIELD_NUMBER: builtins.int
            WIDTH_FIELD_NUMBER: builtins.int
            data: builtins.bytes
            width: builtins.int
            def __init__(
                self,
                *,
                data: builtins.bytes = ...,
                width: builtins.int = ...,
            ) -> None: ...
            def ClearField(self, field_name: typing_extensions.Literal["data", b"data", "width", b"width"]) -> None: ...

        POSITION_FIELD_NUMBER: builtins.int
        HTML_FIELD_NUMBER: builtins.int
        IMAGE_FIELD_NUMBER: builtins.int
        TYPE_FIELD_NUMBER: builtins.int
        @property
        def position(self) -> global___VecXYZf: ...
        html: builtins.str
        @property
        def image(self) -> global___AddObject.Overlay.Image: ...
        type: global___AddObject.Overlay.CoordinateType.ValueType
        def __init__(
            self,
            *,
            position: global___VecXYZf | None = ...,
            html: builtins.str = ...,
            image: global___AddObject.Overlay.Image | None = ...,
            type: global___AddObject.Overlay.CoordinateType.ValueType = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["Contents", b"Contents", "html", b"html", "image", b"image", "position", b"position"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["Contents", b"Contents", "html", b"html", "image", b"image", "position", b"position", "type", b"type"]) -> None: ...
        def WhichOneof(self, oneof_group: typing_extensions.Literal["Contents", b"Contents"]) -> typing_extensions.Literal["html", "image"] | None: ...

    @typing_extensions.final
    class Mesh(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        POINTS_FIELD_NUMBER: builtins.int
        VERTEX_A_INDEX_FIELD_NUMBER: builtins.int
        VERTEX_B_INDEX_FIELD_NUMBER: builtins.int
        VERTEX_C_INDEX_FIELD_NUMBER: builtins.int
        COLORS_FIELD_NUMBER: builtins.int
        @property
        def points(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___VecXYZf]: ...
        @property
        def vertex_a_index(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
        @property
        def vertex_b_index(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
        @property
        def vertex_c_index(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
        @property
        def colors(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___VecRGBf]: ...
        def __init__(
            self,
            *,
            points: collections.abc.Iterable[global___VecXYZf] | None = ...,
            vertex_a_index: collections.abc.Iterable[builtins.int] | None = ...,
            vertex_b_index: collections.abc.Iterable[builtins.int] | None = ...,
            vertex_c_index: collections.abc.Iterable[builtins.int] | None = ...,
            colors: collections.abc.Iterable[global___VecRGBf] | None = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["colors", b"colors", "points", b"points", "vertex_a_index", b"vertex_a_index", "vertex_b_index", b"vertex_b_index", "vertex_c_index", b"vertex_c_index"]) -> None: ...

    @typing_extensions.final
    class Image(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        DOUBLE_SIDE_FIELD_NUMBER: builtins.int
        UPPER_LEFT_FIELD_NUMBER: builtins.int
        LOWER_LEFT_FIELD_NUMBER: builtins.int
        LOWER_RIGHT_FIELD_NUMBER: builtins.int
        DATA_FIELD_NUMBER: builtins.int
        double_side: builtins.bool
        @property
        def upper_left(self) -> global___VecXYZf: ...
        @property
        def lower_left(self) -> global___VecXYZf: ...
        @property
        def lower_right(self) -> global___VecXYZf: ...
        data: builtins.bytes
        def __init__(
            self,
            *,
            double_side: builtins.bool = ...,
            upper_left: global___VecXYZf | None = ...,
            lower_left: global___VecXYZf | None = ...,
            lower_right: global___VecXYZf | None = ...,
            data: builtins.bytes = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["lower_left", b"lower_left", "lower_right", b"lower_right", "upper_left", b"upper_left"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["data", b"data", "double_side", b"double_side", "lower_left", b"lower_left", "lower_right", b"lower_right", "upper_left", b"upper_left"]) -> None: ...

    LINE_SET_FIELD_NUMBER: builtins.int
    POINT_CLOUD_FIELD_NUMBER: builtins.int
    OVERLAY_FIELD_NUMBER: builtins.int
    MESH_FIELD_NUMBER: builtins.int
    IMAGE_FIELD_NUMBER: builtins.int
    @property
    def line_set(self) -> global___AddObject.LineSet: ...
    @property
    def point_cloud(self) -> global___AddObject.PointCloud: ...
    @property
    def overlay(self) -> global___AddObject.Overlay: ...
    @property
    def mesh(self) -> global___AddObject.Mesh: ...
    @property
    def image(self) -> global___AddObject.Image: ...
    def __init__(
        self,
        *,
        line_set: global___AddObject.LineSet | None = ...,
        point_cloud: global___AddObject.PointCloud | None = ...,
        overlay: global___AddObject.Overlay | None = ...,
        mesh: global___AddObject.Mesh | None = ...,
        image: global___AddObject.Image | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["Object", b"Object", "image", b"image", "line_set", b"line_set", "mesh", b"mesh", "overlay", b"overlay", "point_cloud", b"point_cloud"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["Object", b"Object", "image", b"image", "line_set", b"line_set", "mesh", b"mesh", "overlay", b"overlay", "point_cloud", b"point_cloud"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["Object", b"Object"]) -> typing_extensions.Literal["line_set", "point_cloud", "overlay", "mesh", "image"] | None: ...

global___AddObject = AddObject

@typing_extensions.final
class RemoveObject(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ALL_FIELD_NUMBER: builtins.int
    BY_UUID_FIELD_NUMBER: builtins.int
    all: builtins.bool
    by_uuid: builtins.str
    def __init__(
        self,
        *,
        all: builtins.bool = ...,
        by_uuid: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["Object", b"Object", "all", b"all", "by_uuid", b"by_uuid"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["Object", b"Object", "all", b"all", "by_uuid", b"by_uuid"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["Object", b"Object"]) -> typing_extensions.Literal["all", "by_uuid"] | None: ...

global___RemoveObject = RemoveObject

@typing_extensions.final
class SetKeyEventHandler(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    KEYDOWN_FIELD_NUMBER: builtins.int
    KEYUP_FIELD_NUMBER: builtins.int
    KEYPRESS_FIELD_NUMBER: builtins.int
    keydown: builtins.bool
    keyup: builtins.bool
    keypress: builtins.bool
    def __init__(
        self,
        *,
        keydown: builtins.bool = ...,
        keyup: builtins.bool = ...,
        keypress: builtins.bool = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["Event", b"Event", "keydown", b"keydown", "keypress", b"keypress", "keyup", b"keyup"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["Event", b"Event", "keydown", b"keydown", "keypress", b"keypress", "keyup", b"keyup"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["Event", b"Event"]) -> typing_extensions.Literal["keydown", "keyup", "keypress"] | None: ...

global___SetKeyEventHandler = SetKeyEventHandler

@typing_extensions.final
class SetCameraStateEventHandler(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ADD_WITH_INTERVAL_FIELD_NUMBER: builtins.int
    REMOVE_BY_UUID_FIELD_NUMBER: builtins.int
    REMOVE_ALL_FIELD_NUMBER: builtins.int
    add_with_interval: builtins.float
    remove_by_uuid: builtins.str
    remove_all: builtins.bool
    def __init__(
        self,
        *,
        add_with_interval: builtins.float = ...,
        remove_by_uuid: builtins.str = ...,
        remove_all: builtins.bool = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["Action", b"Action", "add_with_interval", b"add_with_interval", "remove_all", b"remove_all", "remove_by_uuid", b"remove_by_uuid"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["Action", b"Action", "add_with_interval", b"add_with_interval", "remove_all", b"remove_all", "remove_by_uuid", b"remove_by_uuid"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["Action", b"Action"]) -> typing_extensions.Literal["add_with_interval", "remove_by_uuid", "remove_all"] | None: ...

global___SetCameraStateEventHandler = SetCameraStateEventHandler

@typing_extensions.final
class VecXYZf(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    X_FIELD_NUMBER: builtins.int
    Y_FIELD_NUMBER: builtins.int
    Z_FIELD_NUMBER: builtins.int
    x: builtins.float
    y: builtins.float
    z: builtins.float
    def __init__(
        self,
        *,
        x: builtins.float = ...,
        y: builtins.float = ...,
        z: builtins.float = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["x", b"x", "y", b"y", "z", b"z"]) -> None: ...

global___VecXYZf = VecXYZf

@typing_extensions.final
class VecRGBf(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    R_FIELD_NUMBER: builtins.int
    G_FIELD_NUMBER: builtins.int
    B_FIELD_NUMBER: builtins.int
    r: builtins.float
    g: builtins.float
    b: builtins.float
    def __init__(
        self,
        *,
        r: builtins.float = ...,
        g: builtins.float = ...,
        b: builtins.float = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["b", b"b", "g", b"g", "r", b"r"]) -> None: ...

global___VecRGBf = VecRGBf

@typing_extensions.final
class SetCustomControl(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing_extensions.final
    class Slider(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        NAME_FIELD_NUMBER: builtins.int
        MIN_FIELD_NUMBER: builtins.int
        MAX_FIELD_NUMBER: builtins.int
        STEP_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        name: builtins.str
        min: builtins.float
        max: builtins.float
        step: builtins.float
        value: builtins.float
        def __init__(
            self,
            *,
            name: builtins.str | None = ...,
            min: builtins.float | None = ...,
            max: builtins.float | None = ...,
            step: builtins.float | None = ...,
            value: builtins.float | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["_max", b"_max", "_min", b"_min", "_name", b"_name", "_step", b"_step", "_value", b"_value", "max", b"max", "min", b"min", "name", b"name", "step", b"step", "value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["_max", b"_max", "_min", b"_min", "_name", b"_name", "_step", b"_step", "_value", b"_value", "max", b"max", "min", b"min", "name", b"name", "step", b"step", "value", b"value"]) -> None: ...
        @typing.overload
        def WhichOneof(self, oneof_group: typing_extensions.Literal["_max", b"_max"]) -> typing_extensions.Literal["max"] | None: ...
        @typing.overload
        def WhichOneof(self, oneof_group: typing_extensions.Literal["_min", b"_min"]) -> typing_extensions.Literal["min"] | None: ...
        @typing.overload
        def WhichOneof(self, oneof_group: typing_extensions.Literal["_name", b"_name"]) -> typing_extensions.Literal["name"] | None: ...
        @typing.overload
        def WhichOneof(self, oneof_group: typing_extensions.Literal["_step", b"_step"]) -> typing_extensions.Literal["step"] | None: ...
        @typing.overload
        def WhichOneof(self, oneof_group: typing_extensions.Literal["_value", b"_value"]) -> typing_extensions.Literal["value"] | None: ...

    @typing_extensions.final
    class CheckBox(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        NAME_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        name: builtins.str
        value: builtins.bool
        def __init__(
            self,
            *,
            name: builtins.str | None = ...,
            value: builtins.bool | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["_name", b"_name", "_value", b"_value", "name", b"name", "value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["_name", b"_name", "_value", b"_value", "name", b"name", "value", b"value"]) -> None: ...
        @typing.overload
        def WhichOneof(self, oneof_group: typing_extensions.Literal["_name", b"_name"]) -> typing_extensions.Literal["name"] | None: ...
        @typing.overload
        def WhichOneof(self, oneof_group: typing_extensions.Literal["_value", b"_value"]) -> typing_extensions.Literal["value"] | None: ...

    @typing_extensions.final
    class TextBox(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        NAME_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        name: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            name: builtins.str | None = ...,
            value: builtins.str | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["_name", b"_name", "_value", b"_value", "name", b"name", "value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["_name", b"_name", "_value", b"_value", "name", b"name", "value", b"value"]) -> None: ...
        @typing.overload
        def WhichOneof(self, oneof_group: typing_extensions.Literal["_name", b"_name"]) -> typing_extensions.Literal["name"] | None: ...
        @typing.overload
        def WhichOneof(self, oneof_group: typing_extensions.Literal["_value", b"_value"]) -> typing_extensions.Literal["value"] | None: ...

    @typing_extensions.final
    class SelectBox(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        NAME_FIELD_NUMBER: builtins.int
        ITEMS_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        name: builtins.str
        @property
        def items(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
        value: builtins.str
        def __init__(
            self,
            *,
            name: builtins.str | None = ...,
            items: collections.abc.Iterable[builtins.str] | None = ...,
            value: builtins.str | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["_name", b"_name", "_value", b"_value", "name", b"name", "value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["_name", b"_name", "_value", b"_value", "items", b"items", "name", b"name", "value", b"value"]) -> None: ...
        @typing.overload
        def WhichOneof(self, oneof_group: typing_extensions.Literal["_name", b"_name"]) -> typing_extensions.Literal["name"] | None: ...
        @typing.overload
        def WhichOneof(self, oneof_group: typing_extensions.Literal["_value", b"_value"]) -> typing_extensions.Literal["value"] | None: ...

    @typing_extensions.final
    class Button(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        NAME_FIELD_NUMBER: builtins.int
        name: builtins.str
        def __init__(
            self,
            *,
            name: builtins.str | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["_name", b"_name", "name", b"name"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["_name", b"_name", "name", b"name"]) -> None: ...
        def WhichOneof(self, oneof_group: typing_extensions.Literal["_name", b"_name"]) -> typing_extensions.Literal["name"] | None: ...

    @typing_extensions.final
    class ColorPicker(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        NAME_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        name: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            name: builtins.str | None = ...,
            value: builtins.str | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["_name", b"_name", "_value", b"_value", "name", b"name", "value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["_name", b"_name", "_value", b"_value", "name", b"name", "value", b"value"]) -> None: ...
        @typing.overload
        def WhichOneof(self, oneof_group: typing_extensions.Literal["_name", b"_name"]) -> typing_extensions.Literal["name"] | None: ...
        @typing.overload
        def WhichOneof(self, oneof_group: typing_extensions.Literal["_value", b"_value"]) -> typing_extensions.Literal["value"] | None: ...

    TARGET_FIELD_NUMBER: builtins.int
    SLIDER_FIELD_NUMBER: builtins.int
    CHECKBOX_FIELD_NUMBER: builtins.int
    TEXTBOX_FIELD_NUMBER: builtins.int
    SELECTBOX_FIELD_NUMBER: builtins.int
    BUTTON_FIELD_NUMBER: builtins.int
    COLOR_PICKER_FIELD_NUMBER: builtins.int
    target: builtins.str
    @property
    def slider(self) -> global___SetCustomControl.Slider: ...
    @property
    def checkbox(self) -> global___SetCustomControl.CheckBox: ...
    @property
    def textbox(self) -> global___SetCustomControl.TextBox: ...
    @property
    def selectbox(self) -> global___SetCustomControl.SelectBox: ...
    @property
    def button(self) -> global___SetCustomControl.Button: ...
    @property
    def color_picker(self) -> global___SetCustomControl.ColorPicker: ...
    def __init__(
        self,
        *,
        target: builtins.str = ...,
        slider: global___SetCustomControl.Slider | None = ...,
        checkbox: global___SetCustomControl.CheckBox | None = ...,
        textbox: global___SetCustomControl.TextBox | None = ...,
        selectbox: global___SetCustomControl.SelectBox | None = ...,
        button: global___SetCustomControl.Button | None = ...,
        color_picker: global___SetCustomControl.ColorPicker | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["Control", b"Control", "button", b"button", "checkbox", b"checkbox", "color_picker", b"color_picker", "selectbox", b"selectbox", "slider", b"slider", "textbox", b"textbox"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["Control", b"Control", "button", b"button", "checkbox", b"checkbox", "color_picker", b"color_picker", "selectbox", b"selectbox", "slider", b"slider", "target", b"target", "textbox", b"textbox"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["Control", b"Control"]) -> typing_extensions.Literal["slider", "checkbox", "textbox", "selectbox", "button", "color_picker"] | None: ...

global___SetCustomControl = SetCustomControl



# pylint: disable=W0611
import multiprocessing  # only for type annotations

import pkgutil
import asyncio
import threading

from http.server import BaseHTTPRequestHandler, HTTPServer
from os.path import join, splitext
from typing import Union, Optional

# pylint: disable=E1101
import websockets
import websockets.server

EXT_TO_MIME = {
    "js": "application/javascript",
    "html": "text/html",
    "css": "text/css",
}


def _MakePointCloudViewerHTTPRequestHandler(websocket_port: int, host: str):
    class _PointCloudViewerHTTPRequestHandler(BaseHTTPRequestHandler):
        def do_GET(self) -> None:
            if self.path == "/":
                self.redirect("index.html")
            elif self.path == "/websocket_url":
                self.send_response(200)
                self.end_headers()
                self.wfile.write(
                    f"ws://{host}:{websocket_port}".encode("utf-8")
                )
            else:
                path = join("/public/", "./"+self.path)
                data: Union[bytes, None] = None
                try:
                    data = pkgutil.get_data("cumo", path)
                except FileNotFoundError:
                    self.send_error(404)
                if data is not None:
                    self.send_response(200)
                    ext = splitext(self.path)[1][1:]
                    if ext in EXT_TO_MIME:
                        self.send_header("content-type", EXT_TO_MIME[ext])
                    self.end_headers()
                    self.wfile.write(data)
                else:
                    self.send_error(404)

        # pylint: disable=W0622
        def log_message(self, format: str, *args) -> None:
            pass

        def redirect(self, path: str):
            self.send_response(301)
            location = path
            self.send_header(
                "Location", location
            )
            self.end_headers()
    return _PointCloudViewerHTTPRequestHandler


def multiprocessing_worker(
    host: str,
    websocket_port: int,
    http_port: int,
    websocket_broadcasting_queue: "multiprocessing.Queue[str]",
    websocket_message_queue: "multiprocessing.Queue[bytes]",
):
    websocket_connection: Optional[websockets.server.WebSocketServerProtocol] = None

    async def __broadcast():
        nonlocal websocket_connection
        loop = asyncio.get_running_loop()
        try:
            while True:
                data = await loop.run_in_executor(None, websocket_broadcasting_queue.get)
                await websocket_connection.send(data)
        except asyncio.CancelledError:
            pass

    async def __websocket_handler(websocket: websockets.server.WebSocketServerProtocol, _path: str):
        nonlocal websocket_connection
        if websocket_connection is not None:
            await websocket.close()
            return
        websocket_connection = websocket
        loop = asyncio.get_running_loop()
        broadcast_task = loop.create_task(__broadcast())
        try:
            msg: Union[str, bytes]
            async for msg in websocket:
                assert isinstance(msg, bytes)
                websocket_message_queue.put(msg)
        finally:
            websocket_connection = None
            broadcast_task.cancel()

    loop = asyncio.get_event_loop()
    http_server = HTTPServer(
        ("", http_port),
        _MakePointCloudViewerHTTPRequestHandler(websocket_port=websocket_port, host=host),
    )

    start_server = websockets.server.serve(__websocket_handler,
                                           host="",
                                           port=websocket_port,
                                           max_size=None,
                                           ping_timeout=60,
                                           )
    loop.run_until_complete(start_server)

    threads = [
        threading.Thread(
            target=loop.run_forever
        ),
        threading.Thread(
            target=http_server.serve_forever
        )
    ]
    thread: threading.Thread
    for thread in threads:
        thread.setDaemon(True)
        thread.start()

    for thread in threads:
        thread.join()



from __future__ import annotations  # Postponed Evaluation of Annotations

from typing import TYPE_CHECKING
from uuid import uuid4

from cumo._internal.protobuf import server_pb2

if TYPE_CHECKING:
    from cumo import PointCloudViewer


def stop_render(self: PointCloudViewer):
    """クライアントの描画を停止させる。また、カメラコントロールも無効化される。
    """
    obj = server_pb2.ServerCommand()
    obj.set_enable = False

    uuid = uuid4()
    self._send_data(obj, uuid)
    ret = self._wait_until(uuid)
    if ret.result.HasField("failure"):
        raise RuntimeError(ret.result.failure)


def resume_render(self: PointCloudViewer):
    """クライアントの描画を再開させる。カメラもマウスによって操作できるようになる。
    """
    obj = server_pb2.ServerCommand()
    obj.set_enable = True

    uuid = uuid4()
    self._send_data(obj, uuid)
    ret = self._wait_until(uuid)
    if ret.result.HasField("failure"):
        raise RuntimeError(ret.result.failure)
